#include "controller.h"
#include "box.h"
#include "wiimote.h"
#include "timer.h"

Music *music;
int last_sync_point = -1;

#define MAX_GROOVIES 35
Groovy groovies[MAX_GROOVIES];

int get_groovy_index(int shape, int level)
{
  return shape * 7 + level;
}

void spawn_groovy(int shape, int level)
{
  int index = get_groovy_index(shape, level);
  Groovy *groovy = &groovies[index];
  groovy->life = 2;
  
  printf("spawn_groovy shape=%d, level=%d\n", shape, level);
  
  throw_particle(index);
  set_volume_on(groovy->track);
}

void init_groovies()
{
  int shape, level;
  Groovy *groovy;
  int index;
  Track *tracks;
  
  for (shape = 0 ; shape < 5 ; shape++)
  {
    switch (shape)
    {
      case 0:
        tracks = music->delay;
        break;
      case 1:
        tracks = music->flute;
        break;
      case 2:
        tracks = music->shamisen;
        break;
      case 3:
        tracks = music->organ;
        break;
        
      case 4:
        tracks = music->bass;
        break;
    }
    
    for (level = 0; level < 7; level++)
    {
      index = get_groovy_index(shape, level);
      groovy = &groovies[index];
      groovy->track = &tracks[level];
      groovy->particle = index;
      groovy->life = 0;
    }
  }
}

void init_controller()
{
  init_groovies();
}

void set_music(Music *new_music)
{
  music = new_music;
}

void tick_sound_event(int numsync)
{
  int i;
  Groovy *groovy;
  
  for (i=0; i<MAX_GROOVIES; i++)
  {
    groovy = &groovies[i];
    
    if (groovy->life)
    {
      groovy->life--;
      if (groovy->life == 0)
      {
        kill_particle(i);
        set_volume_off(groovy->track);
      }
    }
    
  }
  last_sync_point = numsync;
  
  
}

void groove_bass(int bass)
{
  select_track_bass(bass);
}

void key_down_event(SDL_KeyboardEvent *event)
{
  switch(event->keysym.sym)
  {
    /// first line
    case SDLK_F1:
      groove_bass(0);
      break;
      
    case SDLK_F2:
      groove_bass(1);
      break;
      
    case SDLK_F3:
      groove_bass(2);
      break;
      
    case SDLK_F4:
      groove_bass(3);
      break;
      
    case SDLK_F5:
      groove_bass(4);
      break;
      
    case SDLK_F6:
      groove_bass(5);
      break;
      
    case SDLK_F7:
      groove_bass(6);
      break;
      
    case SDLK_F8:
      break;
      
    case SDLK_F9:
      break;
      
    case SDLK_F10:
      break;
      
    case SDLK_F11:
      break;
      
    case SDLK_F12:
      break;
      
    /// second line
    case SDLK_a:
      spawn_groovy(0, 0);
      break;
      
    case SDLK_z:
      spawn_groovy(0, 1);
      break;
      
    case SDLK_e:
      spawn_groovy(0, 2);
      break;
      
    case SDLK_r:
      spawn_groovy(0, 3);
      break;
      
    case SDLK_t:
      spawn_groovy(0, 4);
      break;
      
    case SDLK_y:
      spawn_groovy(0, 5);
      break;
      
    case SDLK_u:
       spawn_groovy(0, 6);
     break;
      
    case SDLK_i:
      spawn_groovy(0, 7);
      break;
      
    case SDLK_o:
      spawn_groovy(3, 0);
      break;
      
    case SDLK_p:
      spawn_groovy(3, 2);
      break;
   
   /// third line
    case SDLK_q:
      spawn_groovy(1, 0);
      break;
      
    case SDLK_s:
      spawn_groovy(1, 1);
      break;
      
    case SDLK_d:
      spawn_groovy(1,2);
      break;
      
    case SDLK_f:
      spawn_groovy(1,3);
      break;
      
    case SDLK_g:
      spawn_groovy(1,4);
      break;
      
    case SDLK_h:
      spawn_groovy(1,5);
      break;
      
    case SDLK_j:
      spawn_groovy(1,6);
      break;
      
    case SDLK_k:
      spawn_groovy(3, 3);
      break;
      
    case SDLK_l:
      spawn_groovy(3, 4);
      break;
    
    case SDLK_m:
      spawn_groovy(3, 5);
      break;
   
    /// fourth line
    case SDLK_w:
      spawn_groovy(2, 0);
      break;
      
    case SDLK_x:
      spawn_groovy(2, 1);
      break;
      
    case SDLK_c:
      spawn_groovy(2, 2);
      break;
      
    case SDLK_v:
      spawn_groovy(2, 3);
      break;
      
    case SDLK_b:
      spawn_groovy(2, 4);
      break;
      
    case SDLK_n:
      spawn_groovy(2, 5);
      break;
      
    case SDLK_COMMA:
      spawn_groovy(2, 6);
      break;
      
    case SDLK_SEMICOLON:
      break;
      
    case SDLK_COLON:
      spawn_groovy(3,6);
      break;
    
    case SDLK_EXCLAIM:
      break;
   
   /// other
    case SDLK_RETURN:
      throw_particle(0);
      break;
      
    /// another keys
    default:
      break;
  }
}
void key_up_event(SDL_KeyboardEvent *event)
{
}

static float max_accel = 0;
static int last_throw_ticks = 0;
#define MAX_ACCELS 10
static float accels[MAX_ACCELS];
float max_sum = 0;

void wiimote_event(struct wiimote_t *wiimote)
{
  struct nunchuk_t *nunchuk = &wiimote->exp.nunchuk;
  float accel;
  int shape;
  float sum;
  int i;
  int level;
  Track *tracks;
  
  accel = wiimote->gforce.x * wiimote->gforce.x +
    wiimote->gforce.y * wiimote->gforce.y +
    wiimote->gforce.z * wiimote->gforce.z;
  
  memcpy(accels, accels + 1, sizeof(float) * (MAX_ACCELS - 1));
  accels[MAX_ACCELS - 1] = accel;
  
  sum = 0;
  for (i=0; i<MAX_ACCELS; i++)
    sum += accels[i];
  //printf("sum accel %f\n", sum);
  
  
  if (sum > max_accel)
    max_accel = sum;
    
  if (IS_JUST_PRESSED(wiimote, WIIMOTE_BUTTON_A))
    max_accel = 0;
    
  //printf("%f\t%f\n", sum, max_accel);
  
  if (last_throw_ticks != 0)
  {
    if (current_ticks - last_throw_ticks > 300)
      last_throw_ticks = 0;
  } else
  {
    if (sum > max_sum)
      max_sum = sum;
    
    if (sum < max_sum / 2.0)
    {
      sum = max_sum;
      max_sum = 0;
      if (sum > 60)
      {
        level = (sum - 60) / (400 - 60) * 7;
        if (level > 6)
          level = 6;
        
        if (IS_PRESSED(wiimote, WIIMOTE_BUTTON_B))
          spawn_groovy(0, level);
        if (IS_PRESSED(wiimote, WIIMOTE_BUTTON_A))
          spawn_groovy(1, level);
        if (IS_PRESSED(nunchuk, NUNCHUK_BUTTON_C))
          spawn_groovy(2, level);
        if (IS_PRESSED(nunchuk, NUNCHUK_BUTTON_Z))
          spawn_groovy(3, level);
        
        last_throw_ticks = current_ticks;
      }
    }
  }

  if (!isnan(wiimote1->exp.nunchuk.js.ang))
  {
    groove_bass(floor(wiimote1->exp.nunchuk.js.ang / 360.0 * 7));
  }
}
