#include <stdlib.h>
#include <stdio.h>
#include <SDL/SDL.h>

#include "sound.h"
#include "graphic.h"
#include "box.h"
#include "timer.h"
#include "wiimote.h"

#define TRUE 1
#define FALSE 0

#define TICK_INTERVAL     10

int running;

/**
 * TimeLeft returns time left to next frame
 **/
Uint32 TimeLeft(void)
{
  static Uint32 next_time = 0;
  Uint32 now;
  
  now = SDL_GetTicks();
  if ( next_time <= now ) {
    next_time = now + TICK_INTERVAL;
    return(0);
  }
  return(next_time - now);
}


int main(int argc, char *argv[])
{
  SDL_Event event;
  int intro_ball = 1;
  
  init_wiimote();
  init_graphic_system();
  init_box();
  
  clear_screen();
  throw_particle_intro();
  draw_loading();
  draw_loading_wiimote();
  draw_title();
  update_graphic_system();
  
  SDL_Delay(100);
  
  init_sound_system();
  
  init_timer();
  
  Music *music = init_music();
  
  set_music(music);
  
  init_controller();
  SDL_Delay(100);
  
  play_music();
  
  running = TRUE;
  while (running == TRUE)
  {
    update_timer();
    update_music();
    update_wiimote();
    
    clear_screen();
    
    if (intro_ball)
    {
      if (wiimote1)
      {
        intro_ball = 0;
        kill_particle_intro();
      } else
      {
        draw_title();
        draw_loading_wiimote();
      } 
    }

    
    while (SDL_PollEvent(&event))
    {
      switch(event.type)
      {
        case SDL_QUIT:
          running = FALSE;
          break;
        case SDL_KEYDOWN:
          key_down_event(&event.key);
          break;
  
        case SDL_KEYUP:
          switch (event.key.keysym.sym)
          {
            case  SDLK_ESCAPE:
              exit(0);
              //running = FALSE;
              break;
            case  SDLK_HOME:
              running = FALSE;
              break;
            case  SDLK_F11:
              toggle_fullscreen();
            default:
              break;
          }
          break;
      }
    }
    
    if (wiimote_requested_exit())
      running = FALSE;
    
    draw_box();
    update_graphic_system();
    SDL_Delay(TimeLeft());
  }
  
  running = TRUE;
  while (running == TRUE)
  {
    update_music();
    clear_screen();
    draw_credits();
    update_graphic_system();
    
    while (SDL_PollEvent(&event))
    {
      switch(event.type)
      {
        case SDL_QUIT:
          running = FALSE;
          break;
  
        case SDL_KEYUP:
          switch (event.key.keysym.sym)
          {
            case  SDLK_ESCAPE:
              running = FALSE;
              break;
            default:
              break;
          }
          break;
      }
    }
    
    SDL_Delay(TimeLeft());
  }
  
  SDL_Delay(100);
  shutdown_sound_system();
  SDL_Delay(100);
  shutdown_graphic_system();
}
