#include "sound.h"

#define COUNT_CHANNEL 36

#define BEAT_FILENAME "./pistesdemosuisse/beat.ogg"

#define BASS_1_FILENAME "./pistesdemosuisse/basse1.ogg"
#define BASS_2_FILENAME "./pistesdemosuisse/basse2.ogg"
#define BASS_3_FILENAME "./pistesdemosuisse/basse3.ogg"
#define BASS_4_FILENAME "./pistesdemosuisse/basse4.ogg"
#define BASS_5_FILENAME "./pistesdemosuisse/basse5.ogg"
#define BASS_6_FILENAME "./pistesdemosuisse/basse6.ogg"
#define BASS_7_FILENAME "./pistesdemosuisse/basse7.ogg"

#define DELAY_1_FILENAME "./pistesdemosuisse/delay1.ogg"
#define DELAY_2_FILENAME "./pistesdemosuisse/delay2.ogg"
#define DELAY_3_FILENAME "./pistesdemosuisse/delay3.ogg"
#define DELAY_4_FILENAME "./pistesdemosuisse/delay4.ogg"
#define DELAY_5_FILENAME "./pistesdemosuisse/delay5.ogg"
#define DELAY_6_FILENAME "./pistesdemosuisse/delay6.ogg"
#define DELAY_7_FILENAME "./pistesdemosuisse/delay7.ogg"

#define FLUTE_1_FILENAME "./pistesdemosuisse/flute1.ogg"
#define FLUTE_2_FILENAME "./pistesdemosuisse/flute2.ogg"
#define FLUTE_3_FILENAME "./pistesdemosuisse/flute3.ogg"
#define FLUTE_4_FILENAME "./pistesdemosuisse/flute4.ogg"
#define FLUTE_5_FILENAME "./pistesdemosuisse/flute5.ogg"
#define FLUTE_6_FILENAME "./pistesdemosuisse/flute6.ogg"
#define FLUTE_7_FILENAME "./pistesdemosuisse/flute7.ogg"

#define ORGAN_1_FILENAME "./pistesdemosuisse/organ1.ogg"
#define ORGAN_2_FILENAME "./pistesdemosuisse/organ2.ogg"
#define ORGAN_3_FILENAME "./pistesdemosuisse/organ3.ogg"
#define ORGAN_4_FILENAME "./pistesdemosuisse/organ4.ogg"
#define ORGAN_5_FILENAME "./pistesdemosuisse/organ5.ogg"
#define ORGAN_6_FILENAME "./pistesdemosuisse/organ6.ogg"
#define ORGAN_7_FILENAME "./pistesdemosuisse/organ7.ogg"

#define SHAMISEN_1_FILENAME "./pistesdemosuisse/shamisen1.ogg"
#define SHAMISEN_2_FILENAME "./pistesdemosuisse/shamisen2.ogg"
#define SHAMISEN_3_FILENAME "./pistesdemosuisse/shamisen3.ogg"
#define SHAMISEN_4_FILENAME "./pistesdemosuisse/shamisen4.ogg"
#define SHAMISEN_5_FILENAME "./pistesdemosuisse/shamisen5.ogg"
#define SHAMISEN_6_FILENAME "./pistesdemosuisse/shamisen6.ogg"
#define SHAMISEN_7_FILENAME "./pistesdemosuisse/shamisen7.ogg"

void ERRCHECK_DEBUG(FMOD_RESULT result, char *filename, int line)
{
  if (result != FMOD_OK)
  {
    printf("FMOD error %d in %s:%d %s\n", result, filename, line, FMOD_ErrorString(result));
    exit(-1);
  }
}
#define ERRCHECK(result) ERRCHECK_DEBUG(result, __FILE__, __LINE__)

FMOD_SYSTEM *sound_system;

Music music;

/*
FMOD_DSP_COMPRESSOR_THRESHOLD Threshold level (dB) in the range from -60 through 0. The default value is 0.
FMOD_DSP_COMPRESSOR_ATTACK Gain reduction attack time (milliseconds), in the range from 10 through 200. The default value is 50.
FMOD_DSP_COMPRESSOR_RELEASE Gain reduction release time (milliseconds), in the range from 20 through 1000. The default value is 50.
FMOD_DSP_COMPRESSOR_GAINMAKEUP Make-up gain (dB) applied after limiting, in the range from 0 through 30. The default value is 0. 

threshold -0,1
attack 50
release 500
gain_makeup 0
*/
void SetCompressor(FMOD_DSP *dsp, int threshold, int attack, int release, int gain_makeup)
{
  FMOD_RESULT       result;
  
  result = FMOD_DSP_SetParameter(dsp, FMOD_DSP_COMPRESSOR_THRESHOLD, threshold);
  ERRCHECK(result);
  
  result = FMOD_DSP_SetParameter(dsp, FMOD_DSP_COMPRESSOR_ATTACK, attack);
  ERRCHECK(result);
  
  result = FMOD_DSP_SetParameter(dsp, FMOD_DSP_COMPRESSOR_RELEASE, release);
  ERRCHECK(result);
  
  result = FMOD_DSP_SetParameter(dsp, FMOD_DSP_COMPRESSOR_GAINMAKEUP, gain_makeup);
  ERRCHECK(result);
}

/*
FMOD_DSP_HIGHPASS_CUTOFF Highpass cutoff frequency in hz. 1.0 to output 22000.0. Default = 5000.0.
FMOD_DSP_HIGHPASS_RESONANCE Highpass resonance Q value. 1.0 to 10.0. Default = 1.0. 
*/
void SetHighpass(FMOD_DSP *dsp, float cutoff, float resonance)
{
  FMOD_RESULT       result;
  
  result = FMOD_DSP_SetParameter(dsp, FMOD_DSP_HIGHPASS_CUTOFF, cutoff);
  ERRCHECK(result);
  
  result = FMOD_DSP_SetParameter(dsp, FMOD_DSP_HIGHPASS_RESONANCE, resonance);
  ERRCHECK(result);
}

/*
FMOD_DSP_LOWPASS_CUTOFF Lowpass cutoff frequency in hz. 10.0 to 22000.0. Default = 5000.0.
FMOD_DSP_LOWPASS_RESONANCE Lowpass resonance Q value. 1.0 to 10.0. Default = 1.0. 
*/
void SetLowpass(FMOD_DSP *dsp, float cutoff, float resonance)
{
  FMOD_RESULT       result;
  
  result = FMOD_DSP_SetParameter(dsp, FMOD_DSP_LOWPASS_CUTOFF, cutoff);
  ERRCHECK(result);
  
  result = FMOD_DSP_SetParameter(dsp, FMOD_DSP_LOWPASS_RESONANCE, resonance);
  ERRCHECK(result);
}

void UpDspTrack(Track *track, FMOD_DSP *dsp)
{
  int active;
  FMOD_RESULT result;

  result = FMOD_DSP_GetActive(dsp, &active);
  ERRCHECK(result);

  if (!active)
  {
    result = FMOD_Channel_AddDSP(track->channel, dsp, 0);
    ERRCHECK(result);
  }
}

void DownDsp(FMOD_DSP *dsp)
{
  int active;
  FMOD_RESULT result;

  result = FMOD_DSP_GetActive(dsp, &active);
  ERRCHECK(result);

  if (active)
  {
    result = FMOD_DSP_Remove(dsp);
    ERRCHECK(result);
  }
}


void init_fmod(FMOD_SYSTEM **system)
{
  FMOD_RESULT result;
  printf("init...\n");
  result = FMOD_System_Create(system);
  ERRCHECK(result);
  printf("... FMOD system\n");
#ifndef WIN32
  result = FMOD_System_SetOutput(*system, FMOD_OUTPUTTYPE_ESD);
  ERRCHECK(result);
#endif
  printf("... FMOD output\n");
  result = FMOD_System_Init(*system, COUNT_CHANNEL, FMOD_INIT_NORMAL, NULL);
  ERRCHECK(result);
  printf("... FMOD inited\n");
}

void shutdown_fmod(FMOD_SYSTEM *system)
{
  FMOD_System_Close(system);
}
 
void init_sound_system()                   // initialize fmod and driver system
{
  init_fmod(&sound_system);
}

void shutdown_sound_system()
{
  stop_music();
  SDL_Delay(100);
  shutdown_fmod(sound_system);
}

void initEffectSet(FMOD_SYSTEM *system, EffectSet *effects)
{
  FMOD_RESULT result;
  
  /// track's highpass
  result = FMOD_System_CreateDSPByType(system, FMOD_DSP_TYPE_HIGHPASS, &effects->dsphighpass);
  ERRCHECK(result);
  
  /// track's lowpass
  result = FMOD_System_CreateDSPByType(system, FMOD_DSP_TYPE_LOWPASS, &effects->dsplowpass);
  ERRCHECK(result);
}

void play_track(FMOD_SYSTEM *system, Track *track, char *filename)
{
  FMOD_RESULT result;
   //FMOD_CHANNEL    *channel;
  result = FMOD_System_PlaySound(system, FMOD_CHANNEL_FREE, track->sound, 0, &track->channel);
  ERRCHECK(result);
  printf("... track playing\n");
  
  initEffectSet(system, &track->effects);
  printf("... init effects\n");
  
  set_volume_off(track);
  
  /// track's highpass
/*  SetHighpass(track->effects.dsphighpass, HIGHPASS_CUTOFF_MAX, HIGHPASS_RESONANCE_MIN);
  DownDsp(track->effects.dsphighpass);
  
  /// track's lowpass
  SetLowpass(track->effects.dsplowpass, LOWPASS_CUTOFF_MAX, LOWPASS_RESONANCE_MIN);
  DownDsp(track->effects.dsplowpass);
  
  /// track's compressor
  SetCompressor(track->effects.dspcompressor, MUSIC_COMPRESSOR_THRESHOLD, MUSIC_COMPRESSOR_ATTACK, MUSIC_COMPRESSOR_RELEASE, MUSIC_COMPRESSOR_GAIN);
  UpDspTrack(track, track->effects.dspcompressor);*/
}

FMOD_RESULT F_CALLBACK on_sync_point( FMOD_CHANNEL *  channel, FMOD_CHANNEL_CALLBACKTYPE type, int command, unsigned int commanddata1, unsigned int commanddata2)
{
  unsigned int numsync = commanddata1;
  tick_sound_event(numsync);
}

void play_music()
{
  FMOD_RESULT result;
  play_track(sound_system, &music.beat, BEAT_FILENAME);
  set_volume_on(&music.beat);
  
  result = FMOD_Channel_SetCallback( music.beat.channel, FMOD_CHANNEL_CALLBACKTYPE_SYNCPOINT, on_sync_point, NULL);
  ERRCHECK(result);
  
  play_track(sound_system, &music.bass[0], BASS_1_FILENAME);
  play_track(sound_system, &music.bass[1], BASS_2_FILENAME);
  play_track(sound_system, &music.bass[2], BASS_3_FILENAME);
  play_track(sound_system, &music.bass[3], BASS_4_FILENAME);
  play_track(sound_system, &music.bass[4], BASS_5_FILENAME);
  play_track(sound_system, &music.bass[5], BASS_6_FILENAME);
  play_track(sound_system, &music.bass[6], BASS_7_FILENAME);
  
  play_track(sound_system, &music.delay[0], DELAY_1_FILENAME);
  play_track(sound_system, &music.delay[1], DELAY_2_FILENAME);
  play_track(sound_system, &music.delay[2], DELAY_3_FILENAME);
  play_track(sound_system, &music.delay[3], DELAY_4_FILENAME);
  play_track(sound_system, &music.delay[4], DELAY_5_FILENAME);
  play_track(sound_system, &music.delay[5], DELAY_6_FILENAME);
  play_track(sound_system, &music.delay[6], DELAY_7_FILENAME);
  
  play_track(sound_system, &music.flute[0], FLUTE_1_FILENAME);
  play_track(sound_system, &music.flute[1], FLUTE_2_FILENAME);
  play_track(sound_system, &music.flute[2], FLUTE_3_FILENAME);
  play_track(sound_system, &music.flute[3], FLUTE_4_FILENAME);
  play_track(sound_system, &music.flute[4], FLUTE_5_FILENAME);
  play_track(sound_system, &music.flute[5], FLUTE_6_FILENAME);
  play_track(sound_system, &music.flute[6], FLUTE_7_FILENAME);
    
  play_track(sound_system, &music.organ[0], ORGAN_1_FILENAME);
  play_track(sound_system, &music.organ[1], ORGAN_2_FILENAME);
  play_track(sound_system, &music.organ[2], ORGAN_3_FILENAME);
  play_track(sound_system, &music.organ[3], ORGAN_4_FILENAME);
  play_track(sound_system, &music.organ[4], ORGAN_5_FILENAME);
  play_track(sound_system, &music.organ[5], ORGAN_6_FILENAME);
  play_track(sound_system, &music.organ[6], ORGAN_7_FILENAME);
  
  play_track(sound_system, &music.shamisen[0], SHAMISEN_1_FILENAME);
  play_track(sound_system, &music.shamisen[1], SHAMISEN_2_FILENAME);
  play_track(sound_system, &music.shamisen[2], SHAMISEN_3_FILENAME);
  play_track(sound_system, &music.shamisen[3], SHAMISEN_4_FILENAME);
  play_track(sound_system, &music.shamisen[4], SHAMISEN_5_FILENAME);
  play_track(sound_system, &music.shamisen[5], SHAMISEN_6_FILENAME);
  play_track(sound_system, &music.shamisen[6], SHAMISEN_7_FILENAME);
}

void initSyncPoints(Music *music, FMOD_SOUND *sound)
{
  FMOD_RESULT result;
  
  char *name = "tick";
  int time = 0;
  FMOD_SYNCPOINT *point[1];
  
  result = FMOD_Sound_AddSyncPoint(sound, time, FMOD_TIMEUNIT_MS, name, point);
  ERRCHECK(result);
}

void initTrack(FMOD_SYSTEM *system, Track *track, char *filename)
{
  FMOD_RESULT result;
  
  //FMOD_SOUND      *sound;
  result = FMOD_System_CreateSound(system, filename, FMOD_HARDWARE | FMOD_LOOP_NORMAL | FMOD_2D, 0, &track->sound );
  ERRCHECK(result);
  printf("... track '%s' loaded\n", filename);
  
  //float            volume;
  track->volume = 1.0f;
}

Music* init_music()          // initialize tracks and music
{
  FMOD_RESULT result;
  
  initTrack(sound_system, &music.beat, BEAT_FILENAME);
  initSyncPoints(sound_system, music.beat.sound);
  
  result = FMOD_System_CreateChannelGroup(sound_system, "tune_plus_bass", &music.tune_plus_bass);
  ERRCHECK(result);
  
  result = FMOD_System_CreateChannelGroup(sound_system, "all_flute", &music.all_flute);
  ERRCHECK(result);
  
  result = FMOD_System_CreateChannelGroup(sound_system, "all_shamisen", &music.all_shamisen);
  ERRCHECK(result);
  
  result = FMOD_System_CreateChannelGroup(sound_system, "all_organ", &music.all_organ);
  ERRCHECK(result);
  
  result = FMOD_System_CreateChannelGroup(sound_system, "all_delay", &music.all_delay);
  ERRCHECK(result);

  initTrack(sound_system, &music.bass[0], BASS_1_FILENAME);
  initTrack(sound_system, &music.bass[1], BASS_2_FILENAME);
  initTrack(sound_system, &music.bass[2], BASS_3_FILENAME);
  initTrack(sound_system, &music.bass[3], BASS_4_FILENAME);
  initTrack(sound_system, &music.bass[4], BASS_5_FILENAME);
  initTrack(sound_system, &music.bass[5], BASS_6_FILENAME);
  initTrack(sound_system, &music.bass[6], BASS_7_FILENAME);
  
  initTrack(sound_system, &music.delay[0], DELAY_1_FILENAME);
  initTrack(sound_system, &music.delay[1], DELAY_2_FILENAME);
  initTrack(sound_system, &music.delay[2], DELAY_3_FILENAME);
  initTrack(sound_system, &music.delay[3], DELAY_4_FILENAME);
  initTrack(sound_system, &music.delay[4], DELAY_5_FILENAME);
  initTrack(sound_system, &music.delay[5], DELAY_6_FILENAME);
  initTrack(sound_system, &music.delay[6], DELAY_7_FILENAME);
  
  initTrack(sound_system, &music.flute[0], FLUTE_1_FILENAME);
  initTrack(sound_system, &music.flute[1], FLUTE_2_FILENAME);
  initTrack(sound_system, &music.flute[2], FLUTE_3_FILENAME);
  initTrack(sound_system, &music.flute[3], FLUTE_4_FILENAME);
  initTrack(sound_system, &music.flute[4], FLUTE_5_FILENAME);
  initTrack(sound_system, &music.flute[5], FLUTE_6_FILENAME);
  initTrack(sound_system, &music.flute[6], FLUTE_7_FILENAME);
    
  initTrack(sound_system, &music.organ[0], ORGAN_1_FILENAME);
  initTrack(sound_system, &music.organ[1], ORGAN_2_FILENAME);
  initTrack(sound_system, &music.organ[2], ORGAN_3_FILENAME);
  initTrack(sound_system, &music.organ[3], ORGAN_4_FILENAME);
  initTrack(sound_system, &music.organ[4], ORGAN_5_FILENAME);
  initTrack(sound_system, &music.organ[5], ORGAN_6_FILENAME);
  initTrack(sound_system, &music.organ[6], ORGAN_7_FILENAME);
  
  initTrack(sound_system, &music.shamisen[0], SHAMISEN_1_FILENAME);
  initTrack(sound_system, &music.shamisen[1], SHAMISEN_2_FILENAME);
  initTrack(sound_system, &music.shamisen[2], SHAMISEN_3_FILENAME);
  initTrack(sound_system, &music.shamisen[3], SHAMISEN_4_FILENAME);
  initTrack(sound_system, &music.shamisen[4], SHAMISEN_5_FILENAME);
  initTrack(sound_system, &music.shamisen[5], SHAMISEN_6_FILENAME);
  initTrack(sound_system, &music.shamisen[6], SHAMISEN_7_FILENAME);
  
    // effets sur la globalité du son
  result = FMOD_System_CreateDSPByType(sound_system, FMOD_DSP_TYPE_COMPRESSOR, &music.dspcompressor);
  ERRCHECK(result);
  
  result = FMOD_System_AddDSP(sound_system, music.dspcompressor, 0);
  ERRCHECK(result);
  
  SetCompressor(music.dspcompressor, MUSIC_COMPRESSOR_THRESHOLD, MUSIC_COMPRESSOR_ATTACK, MUSIC_COMPRESSOR_RELEASE, MUSIC_COMPRESSOR_GAIN);
  printf("... main compressor inited\n");
  return &music;
}

void update_music()
{
  FMOD_System_Update(sound_system);
}

void stop_music()
{
  FMOD_Sound_Release(music.beat.sound);
  
  FMOD_Sound_Release(music.bass[0].sound);
  FMOD_Sound_Release(music.bass[1].sound);
  FMOD_Sound_Release(music.bass[2].sound);
  FMOD_Sound_Release(music.bass[3].sound);
  FMOD_Sound_Release(music.bass[4].sound);
  FMOD_Sound_Release(music.bass[5].sound);
  FMOD_Sound_Release(music.bass[6].sound);
  
  FMOD_Sound_Release(music.flute[0].sound);
  FMOD_Sound_Release(music.flute[1].sound);
  FMOD_Sound_Release(music.flute[2].sound);
  FMOD_Sound_Release(music.flute[3].sound);
  FMOD_Sound_Release(music.flute[4].sound);
  FMOD_Sound_Release(music.flute[5].sound);
  FMOD_Sound_Release(music.flute[6].sound);
    
  FMOD_Sound_Release(music.organ[0].sound);
  FMOD_Sound_Release(music.organ[1].sound);
  FMOD_Sound_Release(music.organ[2].sound);
  FMOD_Sound_Release(music.organ[3].sound);
  FMOD_Sound_Release(music.organ[4].sound);
  FMOD_Sound_Release(music.organ[5].sound);
  FMOD_Sound_Release(music.organ[6].sound);
  
  FMOD_Sound_Release(music.shamisen[0].sound);
  FMOD_Sound_Release(music.shamisen[1].sound);
  FMOD_Sound_Release(music.shamisen[2].sound);
  FMOD_Sound_Release(music.shamisen[3].sound);
  FMOD_Sound_Release(music.shamisen[4].sound);
  FMOD_Sound_Release(music.shamisen[5].sound);
  FMOD_Sound_Release(music.shamisen[6].sound);
}


void set_volume_on(Track *track)
{
  track->volume = 0.8f;
  FMOD_Channel_SetVolume(track->channel, track->volume); 
}

void set_volume_off(Track *track)
{
  track->volume = 0.0f;
  FMOD_Channel_SetVolume(track->channel, track->volume); 
}

void set_lowpass_on(Track *track)
{
  UpDspTrack(track, track->effects.dsplowpass);
}

void set_lowpass_off(Track *track)
{
  DownDsp(track->effects.dsplowpass);
}

void set_highpass_on(Track *track)
{
  UpDspTrack(track, track->effects.dsphighpass);
}

void set_highpass_off(Track *track)
{
  DownDsp(track->effects.dsphighpass);
}

void switch_volume(Track *track)
{
  if (track->volume == 0.0f)
  {
    printf("switch_volume %d to on\n", track);
    set_volume_on(track);
  }
  else
  {
    printf("switch_volume %d to off\n", track);
    set_volume_off(track);
  }
}

void select_track_bass(int index)
{
  int i;
  printf("select_track_bass %d\n", index);
  for (i = 0; i < 7; ++i)
  {
    if (i == index)
    {
      set_volume_on(&music.bass[i]);
    }
    else
    {
      set_volume_off(&music.bass[i]);
    }
  }
}
