#include <stdio.h>
#include <fmodex/fmod.h>
#include <fmodex/fmod_errors.h>

#define MUSIC_COMPRESSOR_THRESHOLD -0.0
#define MUSIC_COMPRESSOR_ATTACK 50
#define MUSIC_COMPRESSOR_RELEASE 500
#define MUSIC_COMPRESSOR_GAIN 0

#define STEP_ADVANCE  150
#define STEP_PAN      0.1

#define EFFECT_COUNT_STEP       100

#define CUTOFF_REAL_MIN      1.0
#define CUTOFF_REAL_MAX      2200.0
#define RESONANCE_REAL_MIN   1.0
#define RESONANCE_REAL_MAX   10.0

#define LOWPASS_CUTOFF_MIN       (0.4 * (CUTOFF_REAL_MAX - CUTOFF_REAL_MIN)) + CUTOFF_REAL_MIN
#define LOWPASS_CUTOFF_MAX       (1.0 * (CUTOFF_REAL_MAX - CUTOFF_REAL_MIN)) + CUTOFF_REAL_MIN
#define LOWPASS_CUTOFF_STEP      (LOWPASS_CUTOFF_MAX - LOWPASS_CUTOFF_MIN) / EFFECT_COUNT_STEP
#define LOWPASS_RESONANCE_MIN    (0.0 * (RESONANCE_REAL_MAX - RESONANCE_REAL_MIN)) + RESONANCE_REAL_MIN
#define LOWPASS_RESONANCE_MAX    (0.14 * (RESONANCE_REAL_MAX - RESONANCE_REAL_MIN)) + RESONANCE_REAL_MIN
#define LOWPASS_RESONANCE_STEP   (LOWPASS_RESONANCE_MAX - LOWPASS_RESONANCE_MIN) / EFFECT_COUNT_STEP

#define HIGHPASS_CUTOFF_MIN       (0.0 * (CUTOFF_REAL_MAX - CUTOFF_REAL_MIN)) + CUTOFF_REAL_MIN
#define HIGHPASS_CUTOFF_MAX       (0.8 * (CUTOFF_REAL_MAX - CUTOFF_REAL_MIN)) + CUTOFF_REAL_MIN
#define HIGHPASS_CUTOFF_STEP      (HIGHPASS_CUTOFF_MAX - HIGHPASS_CUTOFF_MIN) / EFFECT_COUNT_STEP
#define HIGHPASS_RESONANCE_MIN    (0.0 * (RESONANCE_REAL_MAX - RESONANCE_REAL_MIN)) + RESONANCE_REAL_MIN
#define HIGHPASS_RESONANCE_MAX    (0.14 * (RESONANCE_REAL_MAX - RESONANCE_REAL_MIN)) + RESONANCE_REAL_MIN
#define HIGHPASS_RESONANCE_STEP   (HIGHPASS_RESONANCE_MAX - HIGHPASS_RESONANCE_MIN) / EFFECT_COUNT_STEP

typedef struct EffectSet EffectSet;
struct EffectSet {
  FMOD_DSP        *dsphighpass;
  FMOD_DSP        *dsplowpass;
};

typedef struct Track Track;
struct Track {
  FMOD_SOUND      *sound;
  float            volume;
  FMOD_CHANNEL    *channel;
  EffectSet        effects;
};

typedef struct Music Music;
struct Music {
  FMOD_CHANNELGROUP  *all_flute;
  FMOD_CHANNELGROUP  *all_delay;
  FMOD_CHANNELGROUP  *all_organ;
  FMOD_CHANNELGROUP  *all_shamisen;
  FMOD_CHANNELGROUP  *tune_plus_bass;
  
  FMOD_DSP  *dspcompressor_tune_plus_bass;
  FMOD_DSP  *dspcompressor_flute;
  FMOD_DSP  *dspcompressor_delay;
  FMOD_DSP  *dspcompressor_organ;
  FMOD_DSP  *dspcompressor_shamisen;
  
  FMOD_DSP  *dspcompressor;
  
  Track     beat;
  Track     bass[7];
  Track     flute[7];
  Track     delay[7];
  Track     organ[7];
  Track     shamisen[7];
};

void init_sound_system();                   // initialize fmod and driver system
void shutdown_sound_system();

Music* init_music();                 // initialize tracks and music
void play_music();                 // initialize tracks and music
void update_music();
void stop_music();

void set_volume_on(Track *track);
void set_volume_off(Track *track);

void switch_volume(Track *track);
void select_track_bass(int index);

void set_lowpass_on(Track *track);
void set_lowpass_off(Track *track);

void set_highpass_on(Track *track);
void set_highpass_off(Track *track);
